#include <stdio.h>
#include <stdlib.h>
#include "simu.h"

extern int labelcounter;

int main(int argv, char *argc[])
{
    int nout;
    FILE * sim_chroms;
    FILE * snps_file;
    FILE * metadata;
    FILE * outfifteenibd;
    FILE * outnineibd;
    FILE * outposfreq;
    FILE * outfglmap;
    int i,j,k;
    Person * people;
    char ** snps;
    int nsnps, npersons;
    int * snp_pos;
    char ** created;
    int nextindex=0;
    Hashnode ** mytable;
    Chromseglist * curr;
    int fgl_dirty;
    int snp_array_col;
    int * ibdfifteencodes;
    int * ibdninecodes;
/*    double * freqs;
    double numer, denom; */

    if (argv!=5) 
    {
        fprintf(stdout,"Usage: [simulated chroms] [SNPs file] [SNPs metadata] [# of people to create]\n");
        return 1;
    }

    nout = atoi(argc[4]);

    sim_chroms = fopen(argc[1],"r");
    if (!sim_chroms)
    {
        fprintf(stderr,"Error: File %s not found\n",argc[1]);
        return 1;
    }
    snps_file = fopen(argc[2],"r");
    if (!snps_file)
    {
        fprintf(stderr,"Error: File %s not found\n",argc[2]);
        return 1;
    }
    metadata = fopen(argc[3],"r");
    if (!metadata)
    {
        fprintf(stderr,"Error: File %s not found\n",argc[3]);
        return 1;
    }

    outfifteenibd = fopen("outfifteenibd.txt","w");
    if (!outfifteenibd)
    {
        fprintf(stderr,"Error: Could not open file outfifteenibd.txt for writing\n");
        return 1;
    }

    outnineibd = fopen("outnineibd.txt","w");
    if (!outnineibd)
    {
        fprintf(stderr,"Error: Could not open file outnineibd.txt for writing\n");
        return 1;
    }

/*    outposfreq = fopen("outposfreq.txt","w");
    if (!outposfreq)
    {
        fprintf(stderr,"Error: Could not open file outposfreq.txt for writing\n");
        return 1;
    } */

    outfglmap = fopen("outfglmap.txt","w");
    if (!outfglmap)
    {
        fprintf(stderr,"Error: Could not open file outfglmap.txt for writing\n");
        return 1;
    }

    /* read in number of snps and individuals in the real data
     * as well as the snp locations
     * */
    fscanf(metadata,"%d %*s",&npersons);
    fscanf(metadata,"%d %*s",&nsnps);
    snp_pos = malloc(nsnps*sizeof(int));
    for (i=0;i<nsnps;i++)
    {
        fscanf(metadata,"%d",&snp_pos[i]);
    }

    /* read in the REAL snps */
    snps = populate_snps(nsnps,npersons,snps_file);

    /* read in the SIMULATED chromosomes from simulation output */
    people = malloc(nout*sizeof(Person));
    for (i=0;i<nout;i++)
    {
        people[i] = read_person(sim_chroms);
    }

    mytable = make_hashtable();

    /* this will become the output */
    created = malloc(2*nout*sizeof(char *));
    for (i=0;i<nout;i++)
    /* loop through output INDIVIDUALS */
    {
        for (j=0;j<2;j++)
        /* loop through each chromosome */
        {
            created[2*i+j] = malloc(nsnps);
            /* which chromosome are we doing? */
            curr = j ? people[i].paternal : people[i].maternal;
            fgl_dirty = 1;
            for (k=0;k<nsnps;k++)
            /* loop through SNPs */
            {
                while(curr->current->endpos<snp_pos[k])
                {
                    /* fast forward until we're at the right place
                     * on the simulated chromosome; if we move,
                     * we need to look up the new FGL */
                    curr = curr->next;
                    fgl_dirty = 1;
                }
                if (fgl_dirty)
                {
                    /* if we need to check the FGL, look it up in the 
                     * table and possibly update the value there */
                    snp_array_col = get_index(curr->current->fgl,mytable,&nextindex);
                    fgl_dirty = 0;
                    if (snp_array_col >= npersons)
                    {
                        fprintf(stderr,"not enough real founder SNPs!\n");
                        return 1;
                    }
                }
                /* finally, put the char in the output
                 * array */
                created[2*i+j][k] = snps[k][snp_array_col];
            }
        }
    }

    for (i=0;i<nout;i++)
    {
        printf("%d ",people[i].id);
        for (j=0;j<nsnps;j++)
        {
            printf("%c ",created[2*i][j]);
        }
        printf("\n");
        printf("%d ",people[i].id);
        for (j=0;j<nsnps;j++)
        {
            printf("%c ",created[2*i+1][j]);
        }
        printf("\n");
    }
    for (i=0;i<nout;i++)
    {
        for (j=i+1;j<nout;j++)
        {
            ibdfifteencodes = ibd_fifteen(people[i],people[j],nsnps,snp_pos);
            ibdninecodes = ibd_nine(ibdfifteencodes,nsnps);
            fprintf(outfifteenibd,"%d %d ",people[i].id,people[j].id);
            fprintf(outnineibd,"%d %d ",people[i].id,people[j].id);
            for (k=0;k<nsnps;k++)
            {
                fprintf(outfifteenibd,"%d ",ibdfifteencodes[k]);
                fprintf(outnineibd,"%d ",ibdninecodes[k]);
            }
            fprintf(outfifteenibd,"\n");
            fprintf(outnineibd,"\n");
        }
    }

/* doing this in python now 
    freqs = malloc(nsnps*sizeof(double));
    for (i=0;i<nsnps;i++)
    {
        numer = 0;
        denom = 0;
        for (j=0;j<npersons;j++)
        {
            if (snps[i][j]=='1')
            {
                denom++;
                numer++;
            }
            if (snps[i][j]=='2')
                denom++;
        }
        fprintf(outposfreq,"%d %f %f \n",i+1,numer/denom,1-numer/denom);
    }

    print_table(outfglmap,mytable); */

    return 0;
}

